use std::{fs, collections::HashMap};

pub fn solve() {
    let path = "res/07/input.txt";

    let contents = fs::read_to_string(path).expect("I/O error, wrong path?");

    let cont_arr = contents.split("\n").collect::<Vec<&str>>();

    // Hashmap containing all folders and their sizes
    let mut paths = HashMap::<String, i32>::new();
    // Vec containing the current path only
    let mut path = Vec::<&str>::new();

    // Put all paths into the HashMap, add sizes to all parent paths as well
    cont_arr.iter()
        .filter(|line| **line != "") // remove last line
        .for_each(|line| {
            let words = line.split(" ").collect::<Vec<&str>>();
            if line.starts_with("dir ") { } // ignore
            else if line.starts_with("$ ls") { } // ignore
            else if line.starts_with("$ cd ..") { // go folder up by deleting deepest folder
                path.pop();
            }
            else if line.starts_with("$ cd") { // add new folder to path
                path.push(words[2])
            }
            else { // file, add filesize to current and all parent paths
                let size = words[0].parse::<i32>().unwrap();
                for i in 0..path.len() {
                    let current_path = path[0..i+1].join("/");
                    if paths.contains_key(&current_path) {
                        *paths.get_mut(&current_path).unwrap() += size;
                    } else {
                        paths.insert(current_path, size);
                    }
                }
            }
    });
    
    // Sum of all paths lower than 100000 is result
    let res = paths.iter()
        .map(|pair| *pair.1)
        .filter(|size| *size <= 100000)
        .sum::<i32>();

    println!("Result 1: {}", res);

    // Need to calc how much space needs to be freed
    let space_occupied = paths.get("/").unwrap();
    let min_to_delete = 30000000 - (70000000 - space_occupied);

    // Find min folder size higher-or-equal to min_to_delete
    let res = paths.iter()
        .map(|pair| *pair.1)
        .filter(|size| *size >= min_to_delete)
        .min().unwrap();
    
        println!("Result 2: {}", res)
}